/** @jsx React.DOM */

Cosmos.components.DemoHeader = React.createClass({
  /**
   * Just a disclaimer about this being a demo about the Cosmos JavaScript
   * framework consuming TMDb API
   */
  render: function() {
    return (
      React.DOM.p( {className:"demo-header"}, "This is a proof of concept demo for the ",
        React.DOM.strong(null, "Cosmos JavaScript framework,"), " consuming the ",
        React.DOM.a( {href:"https://www.themoviedb.org/"}, "TMDb"), " API. A rudimentary "+
        "example of how easy you explore data with Cosmos, without even controlling the data format. See how fast you navigate between "+
        "Movies and People, especially when going ",
        React.DOM.em(null, "back"), " and ", React.DOM.em(null, "forward"), " into the browser history. ",
        React.DOM.a( {href:"https://github.com/skidding/cosmos", className:"github-link"}, 
        React.DOM.strong(null, "Go to GitHub for specs and source code »"))
      )
    );
  }
});

/** @jsx React.DOM */

var departmentRoleMappings = {
  directing: 'Director',
  writing: 'Writer',
  production: 'Producer',
  acting: 'Actor'
};

Cosmos.components.MovieCredits = React.createClass({
  /**
   * Expected input:
   * {
   *   component: "MovieCredits",
   *   credits: {
   *     acting: [{
   *       id: 3894,
   *       name: "Christian Bale"
   *       order: 0
   *     }],
   *     directing: [{
   *       id: 525,
   *       name: "Christopher Nolan"
   *     }],
   *     production: [{
   *       id: 525,
   *       name: "Christopher Nolan"
   *     }],
   *     writing: [{
   *       id: 527,
   *       name: "Jonathan Nolan"
   *     }]
   *   }
   * }
   */
  mixins: [Cosmos.mixins.Url],
  render: function() {
    return (
      React.DOM.ul( {className:"credits"}, 
        _.map(this.props.credits, function(people, department) {
          return this.renderPeopleLinksForDepartment(people, department);
        }.bind(this))
      )
    );
  },
  renderPeopleLinksForDepartment: function(people, department) {
    // Ignore empty departments and producers
    if (_.isEmpty(people) || department == 'production') {
      return;
    }
    // Only show first 4 actors
    if (department == 'acting') {
      people = _.sortBy(people, function(actor) {
        return actor.order;
      }).slice(0, 4);
    }
    // Add links around people's names
    var creditsWithLinks = _.map(people, function(person) {
      return (
        React.DOM.li( {key:person.id}, 
          React.DOM.a( {href:this.getUrlFromProps({component: 'Person', id: person.id}),
             onClick:this.routeLink}, 
            person.name
          )
        )
      );
    }.bind(this));
    return (
      React.DOM.li( {key:department}, 
        React.DOM.span( {className:"credit-label"}, 
          this.getItemPrefix(departmentRoleMappings[department], people.length)
        ),
        React.DOM.ul( {className:"credit-value"}, 
          creditsWithLinks
        )
      )
    );
  },
  getItemPrefix: function(singular, itemLength) {
    return itemLength > 1 ? singular + 's' : singular;
  }
});

/** @jsx React.DOM */

Cosmos.components.MovieHeader = React.createClass({
  /**
   * Expected input:
   * {
   *   component: "MovieHeader",
   *   title: "The Dark Knight",
   *   year: 2008,
   *   posterPath: "http://image.tmdb.org/t/p/w342/1hRoyzDtpgMU7Dz4JF22RANzQO7.jpg",
   *   credits: {...}
   * }
   */
  render: function() {
    return (
      React.DOM.div( {className:"header"}, 
        React.DOM.div( {className:"title-wrapper"}, 
          React.DOM.img( {className:"poster",
               src:this.props.posterPath,
               alt:""} ),
          React.DOM.h1( {className:"title"}, 
            this.props.title + ' ',
            React.DOM.span( {className:"year"}, this.getYear())
          )
        ),
        Cosmos( {component:"MovieCredits",
                credits:this.props.credits} )
      )
    );
  },
  getYear: function() {
    return this.props.year ? '(' + this.props.year + ')' : null;
  }
});

/** @jsx React.DOM */

Cosmos.components.Movie = React.createClass({
  /**
   * Expected input:
   * {
   *   component: "Movie",
   *   id: 155
   * }
   * Expected state:
   * {
   *   data: {
   *     id: 155,
   *     title: "The Dark Knight",
   *     release_date: "2008-07-18",
   *     poster_path: "/1hRoyzDtpgMU7Dz4JF22RANzQO7.jpg",
   *     genres: [{id: 28, name: "Action"}, {id: 80, name: "Crime"}],
   *     overview: "Batman raises the stakes in his war on crime...",
   *     credits: {
   *       crew: [...],
   *       cast: [...]
   *     },
   *     genres: [{
   *       id: 28,
   *       name: "Action"
   *     }],
   *     similar_movies: {
   *       results: [{
   *         id: 272,
   *         title: "Batman Begins",
   *         poster_path: "/zfVFOo2XCHbeA0mXbst42TAGhfC.jpg",
   *         vote_average: 7,
   *         vote_count: 2492
   *       }]
   *     },
   *     imdb_id: "tt0468569"
   *   }
   * }
   */
  mixins: [Cosmos.mixins.DataFetch,
           Cosmos.mixins.PersistState],
  initialData: {
    credits: {
      crew: [],
      cast: []
    },
    genres: [],
    similar_movies: {
      results: []
    }
  },
  render: function() {
    return (
      React.DOM.div(null, 
        Cosmos( {component:"DemoHeader"} ),
        Cosmos( {component:"MovieHeader",
                title:this.state.data.title || '...',
                year:App.getReleaseYear(this.state.data.release_date),
                posterPath:App.getImagePath(this.state.data.poster_path, 342),
                credits:App.groupCreditsPerDepartments(this.state.data.credits)} ),
        React.DOM.p( {className:"overview"}, 
          React.DOM.strong(null, App.getGenreNames(this.state.data.genres)),
          React.DOM.em(null, this.getMovieOverview(this.state.data.overview))
        ),
        React.DOM.div( {className:"related"}, 
          this.state.data.similar_movies.results.length ?
            React.DOM.p(null, "If you liked ", React.DOM.em(null, this.state.data.title),
               " you might also like...") : '',
          Cosmos( {component:"List",
                  state:{data: this.getPropsForSimilarMoviesList(
                                  this.state.data.similar_movies.results)}} )
        )
      )
    );
  },
  getDataUrl: function(props) {
    return App.getApiPath('movie', props.id, 'credits,similar_movies');
  },
  getMovieOverview: function(overview) {
    return overview ? ' — ' + App.getTextExcerpt(overview, 1000) : '';
  },
  getPropsForSimilarMoviesList: function(movies) {
    var similarMovies = this.filterSimilarMovies(movies);
    return _.map(similarMovies, function(movie) {
      return {
        component: "Thumbnail",
        name: movie.title + ' (' + App.getReleaseYear(movie.release_date) + ')',
        image: App.getImagePath(movie.poster_path, 154),
        linkProps: {
          component: "Movie",
          id: movie.id
        }
      };
    }.bind(this));
  },
  filterSimilarMovies: function(movies) {
    // Can't show thumbnails without images
    movies = _.filter(movies, function(movie) {
      return !!movie.poster_path;
    });
    // Remove unknown or bad movies
    movies = _.filter(movies, function(movie) {
      return movie.vote_count >= 3 && movie.vote_average >= 5;
    });
    // Sort them by vote_average, descending
    movies = _.sortBy(movies, function(movie) {
      return -movie.vote_average;
    });
    return movies;
  }
});

/** @jsx React.DOM */

Cosmos.components.PersonHeader = React.createClass({
  /**
   * Expected input:
   * {
   *   component: "PersonHeader",
   *   name: "Christopher Nolan",
   *   birthday: "July 30, 1970",
   *   birthplace: "London, England, UK",
   *   profilePath: "/7OGmfDF4VHLLgbjxuEwTj3ga0uQ.jpg"
   * }
   */
  render: function() {
    return (
      React.DOM.div( {className:"header"}, 
        React.DOM.div( {className:"title-wrapper"}, 
          React.DOM.img( {className:"poster",
               src:this.props.profilePath,
               alt:""} ),
          React.DOM.h1( {className:"title"}, 
            this.props.name + ' '
          )
        ),
        this.props.birthday ?
          React.DOM.p( {className:"person-bio"}, 
            " Born ", this.props.birthday, " in ",
            React.DOM.strong(null, this.props.birthplace,".")
          ) : ''
      )
    );
  }
});

/** @jsx React.DOM */

var departmentActionMapping = {
  directing: 'directing',
  writing: 'writing',
  production: 'producing',
  acting: 'acting in'
};

Cosmos.components.Person = React.createClass({
  /**
   * Expected input:
   * {
   *   component: "Person",
   *   id: 525
   * }
   * Expected state:
   * {
   *   data: {
   *     id: 525,
   *     name: "Christopher Nolan",
   *     birthday: "1970-07-30",
   *     place_of_birth: "London, England, UK",
   *     profile_path: "/7OGmfDF4VHLLgbjxuEwTj3ga0uQ.jpg",
   *     biography: "Christopher Jonathan James Nolan is a British/American...",
   *     movie_credits: {
   *       crew: [...],
   *       cast: [...]
   *     }
   *     imdb_id: "nm0634240"
   *   }
   * }
   */
  mixins: [Cosmos.mixins.DataFetch,
           Cosmos.mixins.PersistState],
  initialData: {
    movie_credits: {
      crew: [],
      cast: []
    }
  },
  render: function() {
    var groupedCredits =
          App.groupCreditsPerDepartments(this.state.data.movie_credits),
        relevantDepartment = this.getRelevantDepartment(groupedCredits),
        relevantCredits =
          this.filterRelevantCredits(groupedCredits[relevantDepartment]);
    return (
      React.DOM.div(null, 
        Cosmos( {component:"DemoHeader"} ),
        Cosmos( {component:"PersonHeader",
                name:this.state.data.name || '...',
                profilePath:App.getImagePath(this.state.data.profile_path, 342),
                birthday:App.getBirthDay(this.state.data.birthday),
                birthplace:this.state.data.place_of_birth} ),
        React.DOM.p( {className:"overview"}, 
          React.DOM.em(null, App.getTextExcerpt(this.state.data.biography, 600))
        ),
        React.DOM.div( {className:"related"}, 
          relevantCredits.length ?
            React.DOM.p(null, React.DOM.em(null, this.state.data.name), " is known for ",
               departmentActionMapping[relevantDepartment],"...") : '',
          Cosmos( {component:"List",
                  state:{data: this.getPropsForRelevantCredits(relevantCredits)}} )
        )
      )
    );
  },
  getDataUrl: function(props) {
    return App.getApiPath('person', props.id, 'movie_credits');
  },
  getPropsForRelevantCredits: function(relevantCredits) {
    return _.map(relevantCredits, function(movie) {
      return {
        component: "Thumbnail",
        name: movie.title + ' (' + App.getReleaseYear(movie.release_date) + ')',
        image: App.getImagePath(movie.poster_path, 154),
        linkProps: {
          component: "Movie",
          id: movie.id
        }
      };
    }.bind(this));
  },
  filterRelevantCredits: function(movies) {
    // Can't show thumbnails without images
    movies = _.filter(movies, function(movie) {
      return !!movie.poster_path;
    });
    // Don't show movies from the future (can't be known by them...)
    movies = _.filter(movies, function(movie) {
      return new Date(movie.release_date).getTime() < Date.now();
    });
    // Sort them by date, newest first
    movies = _.sortBy(movies, function(movie) {
      return -(new Date(movie.release_date).getTime());
    });
    return movies;
  },
  getRelevantDepartment: function(groupedCredits) {
    var departments = _.keys(groupedCredits);
    // Pick this person's most relevant department by selecting the one
    // that contains the most entries
    return _.sortBy(departments, function(department) {
      return -groupedCredits[department].length;
    })[0];
  }
});

/** @jsx React.DOM */

Cosmos.components.Thumbnail = React.createClass({
  /**
   * Expected input:
   * {
   *   component: "Thumbnail",
   *   name: "The Dark Knight",
   *   image: "http://image.tmdb.org/t/p/w342/1hRoyzDtpgMU7Dz4JF22RANzQO7.jpg",
   *   linkProps: {
   *     component: "Movie",
   *     id: 155
   *   }
   * }
   */
  mixins: [Cosmos.mixins.Url],
  render: function() {
    return (
      React.DOM.a( {className:"thumbnail",
         href:this.getUrlFromProps(this.props.linkProps),
         onClick:this.routeLink}, 
        React.DOM.span( {className:"image",
              style:{backgroundImage: 'url(' + this.props.image + ')'}}
        ),
        this.props.name
      )
    );
  }
});

var App = {
  MOVIEDB_API_ROOT: 'http://api.themoviedb.org/3',
  MOVIEDB_IMG_ROOT: 'http://image.tmdb.org/t/p',
  MOVIEDB_API_KEY: '6cca1ecbcf2b6eec3644b802773e6feb',

  getApiPath: function(entity, id, append_to_response) {
    return App.MOVIEDB_API_ROOT +
           '/' + entity + '/' + id +
           '?append_to_response=' + append_to_response +
           '&api_key=' + App.MOVIEDB_API_KEY;
  },
  getImagePath: function(path, size) {
    return path ? App.MOVIEDB_IMG_ROOT + '/w' + size + path : null;
  },
  getReleaseYear: function(date) {
    return date ? new Date(date).getFullYear() : null;
  },
  getBirthDay: function(date) {
    if (!date) {
      return null;
    }
    var date = new Date(date);
    return this.getMonthName(date.getMonth()) + ' ' + date.getDate() + ', ' +
           date.getFullYear();
  },
  getMonthName: function(monthNumber) {
    return ['January', 'February', 'March', 'April', 'May', 'June', 'July',
      'August', 'September', 'October', 'November', 'December'][monthNumber];
  },
  getGenreNames: function(genres) {
    return _.pluck(genres, 'name').join(', ');
  },
  getTextExcerpt: function(text, limit) {
    if (!text) {
      return '';
    }
    // Get text excerpt while not breaking sentences.
    var sentences = text.split('.'),
        excerpt = '';
    while (sentences.length && excerpt.length < limit) {
      excerpt += sentences.shift() + '.';
    }
    return excerpt;
  },
  groupCreditsPerDepartments: function(credits) {
    return {
      directing: this.filterCrewByDepartment(credits.crew, 'Directing'),
      writing: this.filterCrewByDepartment(credits.crew, 'Writing'),
      production: this.filterCrewByDepartment(credits.crew, 'Production'),
      acting: credits.cast
    };
  },
  filterCrewByDepartment: function(crew, department) {
    var filteredCrew = _.filter(crew, function(person) {
      return person.department == department;
    });
    // Remove duplicate entries when a person has more than one role in the
    // same department
    return _.uniq(filteredCrew, false, function(person) {
      return person.id;
    });
  }
};
